<?php
// $Horde: kronolith/lib/Calendar.php,v 1.00 2002/07/15 14:18:24 jvandal Exp $

/**
 * This class provides an interface to all calendars a user might
 * have. Its methods take care of any site-specific restrictions
 * configured in prefs.php and conf.php.
 *
 * @author  Joel Vandal <jvandal@infoteck.qc.ca>
 * @version $Revision: 1.00 $
 * @since   Horde 3.0
 * @package kronolith.calendar
 */
class Calendar {

    /**
     * Array containing all the user's calendars.
     * @var array $calendars
     */
    var $calendars = array();

    /**
     * A pointer to the user's standard calendar.
     * This one is used by the methods returning values
     * if no other one is specified.
     * @var integer $default
     */
    var $default = 0;

    /**
     * Array containing all of the properties in this calendar,
     * excluding the id.
     * @var array $properties
     */
    var $properties = array();

    /**
     * Reference to the prefs object that this calendar points to.
     * @var object Prefs $prefs
     */
    var $prefs;

    /**
     * Reads all the user's calendars from the prefs object or builds
     * a new calendar from the standard values given in prefs.php.
     *
     * @param optional string $user  If specified, we read another user's
     *                               calendars instead of the current user.
     */
    function Calendar($user = null)
    {
        if (is_null($user)) {
            $this->prefs = &$GLOBALS['prefs'];
            $this->default = $this->prefs->getValue('default_calendar');
            $this->calendars = @unserialize($this->prefs->getValue('calendars'));
        } else {
            global $registry;
            $this->prefs = &Prefs::singleton($GLOBALS['conf']['prefs']['driver'], $registry->getApp(), '', '', $GLOBALS['conf']['prefs']['params']);
            $this->prefs->setDefaults($registry->getParam('fileroot', 'horde') . '/config/prefs.php');
            $this->prefs->setDefaults($registry->getParam('fileroot') . '/config/prefs.php');

            $vals = $this->prefs->getPref($user, array('default_calendar', 'calendars'));
            $this->default = isset($vals['default_calendar']) ? $vals['default_calendar'] : 0;
            $this->calendars = @unserialize($vals['calendars']);
        }

        if (!is_array($this->calendars) || (count($this->calendars) <= 0)) {
            $calendar = array('id' => _("Personal")) ;
//			      'type' => 2);
            foreach ($this->properties as $key) {
                $calendar[$key] = $this->prefs->getValue($key);
            }
	    
            $this->calendars[] = $calendar;
        }
    }

    /**
     * Saves all calendars in the prefs backend.
     */
    function save()
    {
        $this->prefs->setValue('calendars', serialize($this->calendars));
        $this->prefs->setValue('default_calendar', $this->default);
        $this->prefs->store();
    }

    /**
     * Adds a new empty calendar to the array of calendars.
     * @return integer      The pointer to the created calendar
     */
    function add()
    {
        $this->calendars[] = array();
        return count($this->calendars) - 1;
    }

    /**
     * Remove an calendar from the array of calendars
     * @param integer $calendar The pointer to the calendar to be removed
     * @return array            The removed calendar
     */
    function delete($calendar)
    {
        $deleted = array_splice($this->calendars, $calendar, 1);
        foreach ($this->calendars as $id => $null) {
            if ($this->setDefault($id)) {
                break;
            }
        }
        $this->save();
        return $deleted;
    }

    /**
     * Returns a pointer to the current default calendar.
     * @return integer      The pointer to the current default calendar
     */
    function getDefault()
    {
        return $this->default;
    }

    /**
     * Sets the current default calendar.
     * If the calendar doesn't exist, the old default calendar
     * stays the same.
     * @param integer $calendar     The pointer to the new default calendar
     * @return boolean              True on success, false on failure
     */
    function setDefault($calendar)
    {
        if (isset($this->calendars[$calendar])) {
            $this->default = $calendar;
            return true;
        } else {
            return false;
        }
    }

    /**
     * Returns a property from one of the calendars. If this value doesn't exist
     * or is locked, the property is retrieved from the prefs backend.
     * @param string $key           The property to retrieve.
     * @param integer $calendar     (optional) The calendar to retrieve the property from
     * @return mixed                The value of the property.
     */
    function getValue($key, $calendar = null)
    {
        if (!isset($calendar) || !isset($this->calendars[$calendar])) {
            $calendar = $this->default;
        }

        if (!isset($this->calendars[$calendar][$key]) || $this->prefs->isLocked($key)) {
            $val = $this->prefs->getValue($key);
        } else {
            $val = $this->calendars[$calendar][$key];
        }

        return $val;
    }

    /**
     * Returns an array with the specified property from all existing
     * calendars.
     * @param string $key       The property to retrieve.
     * @return array            The array with the values from all calendars
     */
    function getAll($key)
    {

        $list = array();
        foreach ($this->calendars as $calendar => $null) {
            $list[$calendar] = $this->getValue($key, $calendar);
        }
        return $list;
    }

    /**
     * Sets a property with a specified value.
     * @param string $key       The property to set
     * @param mixed $val        The value to which the property should be set
     * @param integer $calendar (optional) The calendar to set the property in
     * @return boolean          True on success, false on failure (property was locked)
     */
    function setValue($key, $val, $calendar = null)
    {
        if (!isset($calendar)) {
            $calendar = $this->default;
        }

        if (!$this->prefs->isLocked($key)) {
            $this->calendars[$calendar][$key] = $val;
            return true;
        } else {
            return false;
        }
    }

    /**
     * Returns true if all properties are locked and therefore nothing
     * in the calendars can be changed.
     * @return boolean          True if all properties are locked, false otherwise
     */
    function isLocked()
    {
        foreach ($this->properties as $key) {
            if (!$this->prefs->isLocked($key)) {
                return false;
            }
        }

        return true;
    }

    /**
     * Returns true if the given address belongs to one of the calendars.
     *
     * @param string $key          The calendar key to search.
     * @param string $value        The value to search for in $key.
     *
     * @return boolean             True if the $value was found in $key.
     */
    function hasValue($key, $valueA)
    {
        $list = $this->getAll($key);
        foreach ($list as $valueB) {
            if (strpos(strtolower($valueA), strtolower($valueB)) !== false) {
                return true;
            }
        }
        return false;
    }

}
?>
